/* Shared library add-on to iptables to add packet log matching support. */
#include <stdio.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <iptables.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_log.h>

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"log v%s options:\n"
" --log-prefix prefix		Prefix log messages with this prefix.\n\n"
" --log-destip destip		Log DestIP.\n\n"
IPTABLES_VERSION);
}

static struct option opts[] = {
	{ .name = "log-prefix", .has_arg = 1, .flag = 0, .val = '1' },
	{ .name = "log-destip", .has_arg = 0, .flag = 0, .val = '2' },
	{ .name = 0 }
};

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      const struct ipt_entry *entry,
      unsigned int *nfcache,
      struct ipt_entry_match **match)
{
	struct ipt_logs_info *info = (struct ipt_logs_info *)(*match)->data;

	switch (c) {
		case '1':
			if (strlen(optarg) > sizeof(info->prefix) - 1)
				exit_error(PARAMETER_PROBLEM,
						"Maximum prefix length %u for --log-prefix",
						(unsigned int)sizeof(info->prefix) - 1);
		
			strcpy(info->prefix, optarg);
			break;
		case '2':	
			strcpy(info->destip, optarg);
			break;
		default:
			return 0;
	}
	return 1;
}

/* Final check; nothing. */
static void
final_check(unsigned int flags)
{

}

/* Prints out the matchinfo. */
static void
print(const struct ipt_ip *ip,
      const struct ipt_entry_match *match,
      int numeric)
{
	struct ipt_logs_info *info = (struct ipt_logs_info *)match->data;
	
	printf("log: ");
	if (strcmp(info->prefix, "") != 0)
		printf("prefix \"%s\" ", info->prefix);

	if (strcmp(info->destip, "") != 0)
		printf("destip \"%s\" ", info->destip);
}

/* Saves the union ipt_matchinfo in parsable form to stdout. */
static void
save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
{
	struct ipt_logs_info *info = (struct ipt_logs_info *)match->data;

	if (strcmp(info->prefix, "") != 0)
		printf("--log-prefix \"%s\" ", info->prefix);

	if (strcmp(info->destip, "") != 0)
		printf("--log-destip \"%s\" ", info->destip);
}

static struct iptables_match match_log = { 
	.next		= NULL,
	.name		= "log",
	.version	= IPTABLES_VERSION,
	.size		= IPT_ALIGN(sizeof(struct ipt_logs_info)),
	.userspacesize	= IPT_ALIGN(sizeof(struct ipt_logs_info)),
	.help		= &help,
	.parse		= &parse,
	.final_check	= &final_check,
	.print		= &print,
	.save		= &save,
	.extra_opts	= opts
};

void _init(void)
{
	register_match(&match_log);
}

