#ifndef _CONFIG_H
#define _CONFIG_H


#define NETWALL_VERSION		"v2.0"


/******************** Interface Definition ***************************/
#define NET_IPC_IFNAME	"eth0" 	/* dhcp, static iface names, such as 'eth0' */
#define NET_PPP_IFNAME	"ppp0"		/* pppoe, l2tp, pptp iface names */
#define NET_LOC_IFNAME	"br0"		/* lan interface names, such as 'br0' */


/***************************************************************/

/*
  * @ SPI Firewall/DoS Protection Implementation in NETFILTER match.
  * @ According to 'Home Wireless Router Engineer Spec v1.6'
  */
#define DNI_SPI_DoS_SUPPORT	1

/*
  * @ For PPTP connection type, we have two interfaces, one is ppp, which is mainly used 
  * @ to send and receive network data; the other is NET_IPC_IFNAME, which is used to
  * @ setup the PPTP session, so the net-wall should do something special.
  */
#define DNI_PPTP_SUPPORT	1

/*
  * @ For meeting NETGEAR's spec, and let the firewall log the information we have interest 
  * @ on, we define a new but have much limited using match 'ipt_logs_info'.
  */
#define DNI_LOG_SUPPORT		1

/*
  * @ For let the UPnP application add & delete the Port-Mapping entry easily, we do it in the
  * @ net-wall as if the demo formats the Port-Mapping parameters as needed. Please read the
  * @ net-wall code for reference.
  */
#define DNI_UPnP_SUPPORT	1

/* @ This is used for UDP NAT 'ipt_conenat_info' @ */
#define DNI_CONENAT_SUPPORT	1

#define DNI_SNATP2P_SUPPORT	1

#define DNI_QoS_SUPPORT	1

/* @ '"Block Site/URL Filtering" @ */
#define DNI_BLOCK_URL_SUPPORT	1

/* @ "Port Triggering" ipt_trigger_info @ */
#define DNI_PORT_TRIGGER_SUPPORT	1

/* @ Linux 2.6 add one data member 'u8 revision' in match & target definition @ */
#define DNI_LINUX_VERSION_2_6	1

#define DNI_DNS_HIJACK_SUPPORT	1

#define DNI_Parental_Control_SUPPORT 1

#define DNI_NETGEAR_REJECT_SUPPORT 1

/******************* Data Type Definition *****************************/
typedef char s8;
typedef unsigned char u8;

typedef short s16;
typedef unsigned short u16;

typedef int s32;
typedef unsigned int u32;

typedef unsigned long long u64;
typedef long long s64;
	
#endif

